@@ -6,11 +6,12 @@ use 5.006;
 use strict;
 use warnings;
 
-our $VERSION = '0.15';
+our $VERSION = '0.16';
 
 use File::Copy;
 use File::Path;
 use Log::Log4perl qw(:easy);
+use Log::Log4perl::Util;
 use LWP::Simple;
 use File::Basename;
 use File::Spec::Functions qw(rel2abs abs2rel);
@@ -20,6 +21,7 @@ use File::Temp;
 
 our $DRY_RUN;
 our $DRY_RUN_MSG;
+our $DATA_SNIPPED_LEN = 60;
 
 dry_run(0);
 
@@ -46,6 +48,7 @@ sysrun untar_in pick ask
 hammer say
 sudo_me bin_find
 fs_read_open fs_write_open pipe_copy
+snip
 );
 
 our %EXPORTABLE = map { $_ => 1 } @EXPORTABLE;
@@ -69,7 +72,7 @@ sub import {
     }
 
     for my $func (keys %tags) {
-        die __PACKAGE__ . 
+        LOGDIE __PACKAGE__ . 
             "doesn't export \"$func\"" unless exists $EXPORTABLE{$func};
         *{"$caller_pkg\::$func"} = *{$func};
     }
@@ -122,6 +125,11 @@ logs everything, but suppresses any write actions. Dry run mode
 is enabled by calling C<Sysadm::Install::dry_run(1)>. To switch
 back to normal, call C<Sysadm::Install::dry_run(0)>.
 
+C<Sysadm::Install> is fully Log4perl-enabled. To start logging, just
+initialize C<Log::Log4perl>. C<Sysadm::Install> acts as a wrapper class,
+meaning that file names and line numbers are reported from the calling
+program's point of view.
+
 =head2 FUNCTIONS
 
 =over 4
@@ -142,6 +150,8 @@ shown below.
 ###############################################
 sub cp {
 ###############################################
+
+    local($Log::Log4perl::caller_depth) += 1;
     INFO "cp $_[0] $_[1] $DRY_RUN_MSG";
     return 1 if $DRY_RUN;
     File::Copy::copy @_ or LOGDIE "Cannot copy $_[0] to $_[1] ($!)";
@@ -158,6 +168,8 @@ Move a file from C<$source> to C<$target>. C<target> can be a directory.
 ###############################################
 sub mv {
 ###############################################
+
+    local($Log::Log4perl::caller_depth) += 1;
     INFO "mv $_[0] $_[1] $DRY_RUN_MSG";
     return 1 if $DRY_RUN;
     File::Copy::move @_ or LOGDIE "Cannot move $_[0] to $_[1] ($!)";
@@ -175,6 +187,8 @@ name returned by C<basename($url)>.
 ###############################################
 sub download {
 ###############################################
+
+    local($Log::Log4perl::caller_depth) += 1;
     INFO "Downloading $_[0] => ", basename($_[0]), " $DRY_RUN_MSG";
     return 1 if $DRY_RUN;
     getstore($_[0], basename($_[0])) or LOGDIE "Cannot download $_[0] ($!)";
@@ -182,29 +196,36 @@ sub download {
 
 =pod
 
-=item C<untar($tgz_file)>
+=item C<untar($tarball)>
 
-Untar the tarball in C<$tgz_file>, which typically adheres to the
-C<someproject-X.XX.tgz> convention. But regardless of whether the 
+Untar the tarball in C<$tarball>, which typically adheres to the
+C<someproject-X.XX.tgz> convention. 
+But regardless of whether the 
 archive actually contains a top directory C<someproject-X.XX>,
 this function will behave if it had one. If it doesn't have one,
 a new directory is created before the unpacking takes place. Unpacks
 the tarball into the current directory, no matter where the tarfile
-is located.
+is located. 
+Please note that if you're
+using a compressed tarball (.tar.gz or .tgz), you'll need
+IO::Zlib installed. 
 
 =cut
 
 ###############################################
 sub untar {
 ###############################################
-    die "untar called without defined tarfile" unless @_ == 1 
-         and defined $_[0];
+    local($Log::Log4perl::caller_depth) += 1;
+
+    LOGDIE "untar called without defined tarfile" unless 
+         @_ == 1 and defined $_[0];
 
     INFO "untar $_[0] $DRY_RUN_MSG";
     return 1 if $DRY_RUN;
 
     my($nice, $topdir, $namedir) = archive_sniff($_[0]);
 
+    check_zlib($_[0]);
     my $arch = Archive::Tar->new($_[0]);
 
     if($nice and $topdir eq $namedir) {
@@ -217,9 +238,10 @@ sub untar {
         rmf($topdir);
             # extract as topdir
         $arch->extract();
-        rename $topdir, $namedir or die "Can't rename $topdir, $namedir";
+        rename $topdir, $namedir or 
+            LOGDIE "Can't rename $topdir, $namedir";
     } else {
-        die "no topdir" unless defined $topdir;
+        LOGDIE "no topdir" unless defined $topdir;
         DEBUG "Not-so-nice archive (no topdir), extracting to subdir $topdir";
         $topdir = basename $topdir;
         rmf($topdir);
@@ -246,6 +268,8 @@ sub untar_in {
 ###############################################
     my($tar_file, $dir) = @_;
 
+    local($Log::Log4perl::caller_depth) += 1;
+
     LOGDIE "not enough arguments" if
       ! defined $tar_file or ! defined $dir;
 
@@ -257,6 +281,8 @@ sub untar_in {
     my $tar_file_abs = rel2abs($tar_file, dirname($tar_file));
 
     cd($dir);
+
+    check_zlib($tar_file_abs);
     my $arch = Archive::Tar->new("$tar_file_abs");
     $arch->extract() or LOGDIE "Extract failed: $!";
     cdback();
@@ -295,11 +321,13 @@ sub pick {
 ##################################################
     my ($prompt, $options, $default) = @_;    
 
+    local($Log::Log4perl::caller_depth) += 1;
+
     my $default_int;
     my %files;
 
     if(@_ != 3 or ref($options) ne "ARRAY") {
-        die "pick called with wrong #/type of args";
+        LOGDIE "pick called with wrong #/type of args";
     }
     
     {
@@ -338,8 +366,10 @@ sub ask {
 ##################################################
     my ($prompt, $default) = @_;    
 
+    local($Log::Log4perl::caller_depth) += 1;
+
     if(@_ != 2) {
-        die "ask() called with wrong # of args";
+        LOGDIE "ask() called with wrong # of args";
     }
 
     print STDERR "$prompt [$default]> ";
@@ -362,6 +392,8 @@ Create a directory of arbitrary depth, just like C<File::Path::mkpath>.
 ###############################################
 sub mkd {
 ###############################################
+
+    local($Log::Log4perl::caller_depth) += 1;
     INFO "mkd @_ $DRY_RUN_MSG";
     return 1 if $DRY_RUN;
     mkpath @_ or LOGDIE "Cannot mkdir @_ ($!)";
@@ -379,6 +411,8 @@ in the shell.
 ###############################################
 sub rmf {
 ###############################################
+
+    local($Log::Log4perl::caller_depth) += 1;
     INFO "rmf $_[0] $DRY_RUN_MSG";
     return 1 if $DRY_RUN;
 
@@ -400,6 +434,8 @@ chdir to the given directory.
 ###############################################
 sub cd {
 ###############################################
+
+    local($Log::Log4perl::caller_depth) += 1;
     INFO "cd $_[0]";
 
     push @DIR_STACK, getcwd();
@@ -417,7 +453,10 @@ chdir back to the last directory before a previous C<cd>.
 ###############################################
 sub cdback {
 ###############################################
-    die "cd stack empty" unless @DIR_STACK;
+
+    local($Log::Log4perl::caller_depth) += 1;
+
+    LOGDIE "cd stack empty" unless @DIR_STACK;
 
     my $old_dir = pop @DIR_STACK;
     INFO "cdback to $old_dir";
@@ -435,6 +474,9 @@ Call C<make> in the shell.
 ###############################################
 sub make {
 ###############################################
+
+    local($Log::Log4perl::caller_depth) += 1;
+
     INFO "make @_ $DRY_RUN_MSG";
     return 1 if $DRY_RUN;
 
@@ -445,11 +487,26 @@ sub make {
 
 =cut
 
+###############################################
+sub check_zlib {
+###############################################
+    my($tar_file) = @_;
+
+    if($tar_file =~ /\.tar\.gz\b|\.tgz\b/ and
+       !Log::Log4perl::Util::module_available("IO::Zlib")) {
+
+        LOGDIE "$tar_file: Compressed tarballs can only be processed",
+               " with IO::Zlib installed.";
+    }
+}
+     
 #######################################
 sub archive_sniff {
 #######################################
     my($name) = @_;
 
+    local($Log::Log4perl::caller_depth) += 1;
+
     DEBUG "Sniffing archive '$name'";
 
     my ($dir) = ($name =~ /(.*?)\.(tar\.gz|tgz|tar)$/);
@@ -461,11 +518,13 @@ sub archive_sniff {
 
     my $topdir;
 
+    check_zlib($name);
+
     my $tar = Archive::Tar->new($name);
 
     my @names = $tar->list_files(["name"]);
 
-    die "Archive $name is empty" unless @names;
+    LOGDIE "Archive $name is empty" unless @names;
 
     (my $archdir = $names[0]) =~ s#/.*##;
 
@@ -509,6 +568,8 @@ sub pie {
 ###############################################
     my($coderef, @files) = @_;
 
+    local($Log::Log4perl::caller_depth) += 1;
+
     for my $file (@files) {
 
         INFO "editing $file in-place $DRY_RUN_MSG";
@@ -547,6 +608,8 @@ sub plough {
 ###############################################
     my($coderef, @files) = @_;
 
+    local($Log::Log4perl::caller_depth) += 1;
+
     for my $file (@files) {
 
         INFO "Ploughing through $file $DRY_RUN_MSG";
@@ -575,7 +638,9 @@ sub slurp {
 ###############################################
     my($file) = @_;
 
-    INFO "slurping data from $file";
+    local($Log::Log4perl::caller_depth) += 1;
+
+    INFO "Slurping data from $file";
 
     local $/ = undef;
 
@@ -583,7 +648,7 @@ sub slurp {
     my $data = <FILE>;
     close FILE;
 
-    DEBUG "Read ", length($data), " bytes from $file";
+    DEBUG "Read ", snip($data, $DATA_SNIPPED_LEN), " from $file";
 
     return $data;
 }
@@ -603,7 +668,9 @@ sub blurt {
 ###############################################
     my($data, $file, $append) = @_;
 
-    INFO(($append ? "appending" : "writing") . " " .
+    local($Log::Log4perl::caller_depth) += 1;
+
+    INFO(($append ? "Appending" : "Writing") . " " .
          length($data) . " bytes to $file $DRY_RUN_MSG");
     return 1 if $DRY_RUN;
 
@@ -611,6 +678,8 @@ sub blurt {
         or LOGDIE "Cannot open $file for writing ($!)";
     print FILE $data;
     close FILE;
+
+    DEBUG "Wrote ", snip($data, $DATA_SNIPPED_LEN), " to $file";
 }
 
 =pod
@@ -661,6 +730,8 @@ sub tap {
 ###############################################
     my(@args) = @_;
 
+    local($Log::Log4perl::caller_depth) += 1;
+
     if($DRY_RUN) {
         INFO "tapping @args $DRY_RUN_MSG";
         return 1;
@@ -833,6 +904,8 @@ sub perm_cp {
     # Lifted from Ben Okopnik's
     # http://www.linuxgazette.com/issue87/misc/tips/cpmod.pl.txt
 
+    local($Log::Log4perl::caller_depth) += 1;
+
     INFO "perm_cp @_ $DRY_RUN_MSG";
     return 1 if $DRY_RUN;
 
@@ -857,6 +930,8 @@ sub perm_get {
 ######################################
     my($filename) = @_;
 
+    local($Log::Log4perl::caller_depth) += 1;
+
     my @stats = (stat $filename)[2,4,5] or
         LOGDIE "Cannot stat $filename ($!)";
 
@@ -880,6 +955,8 @@ sub perm_set {
 ######################################
     my($filename, $perms) = @_;
 
+    local($Log::Log4perl::caller_depth) += 1;
+
     INFO "perm_set $filename (@$perms) $DRY_RUN_MSG";
     return 1 if $DRY_RUN;
 
@@ -904,6 +981,8 @@ sub sysrun {
 ######################################
     my(@cmds) = @_;
 
+    local($Log::Log4perl::caller_depth) += 1;
+
     INFO "sysrun: @cmds $DRY_RUN_MSG";
     return 1 if $DRY_RUN;
 
@@ -928,6 +1007,8 @@ sub hammer {
 
     require Expect;
 
+    local($Log::Log4perl::caller_depth) += 1;
+
     if($DRY_RUN) {
         INFO "Hammer: @cmds $DRY_RUN_MSG";
         return 1 if $DRY_RUN;
@@ -977,6 +1058,8 @@ sub sudo_me {
 ######################################
     my($argv) = @_;
 
+    local($Log::Log4perl::caller_depth) += 1;
+
     if($DRY_RUN) {
         INFO "sudo_me $DRY_RUN_MSG";
         return 1;
@@ -1035,6 +1118,8 @@ sub fs_read_open {
 ######################################
     my($dir) = @_;
 
+    local($Log::Log4perl::caller_depth) += 1;
+
     my $find = bin_find("find");
     LOGDIE "Cannot find 'find'" unless defined $find;
 
@@ -1071,6 +1156,8 @@ sub fs_write_open {
 ######################################
     my($dir) = @_;
 
+    local($Log::Log4perl::caller_depth) += 1;
+
     my $cpio = bin_find("cpio");
     LOGDIE "Cannot find 'cpio'" unless defined $cpio;
 
@@ -1102,6 +1189,8 @@ sub pipe_copy {
 ######################################
     my($in, $out, $bufsize) = @_;
 
+    local($Log::Log4perl::caller_depth) += 1;
+
     $bufsize ||= 4096;
     my $bytes = 0;
 
@@ -1117,6 +1206,61 @@ sub pipe_copy {
 
 =pod
 
+=item C<snip($data, $maxlen)>
+
+Format the data string in C<$data> so that it's only (roughly) $maxlen
+characters long and only contains printable characters.
+
+If C<$data> contains unprintable character's they are replaced by 
+"." (the dot). If C<$data> is longer than C<$maxlen>, it will be
+formatted like
+
+    (22)[abcdef[snip=11]stuvw]
+
+indicating the length of the original string, the beginning, the
+end, and the number of 'snipped' characters.
+
+=cut
+
+###########################################
+sub snip {
+###########################################
+    my($data, $maxlen) = @_;
+
+    if(length $data <= $maxlen) {
+        return lenformat($data);
+    }
+
+    $maxlen = 12 if $maxlen < 12;
+    my $sniplen = int(($maxlen - 8) / 2);
+
+    my $start   = substr($data,  0, $sniplen);
+    my $end     = substr($data, -$sniplen);
+    my $snipped = length($data) - 2*$sniplen;
+
+    return lenformat("$start\[snip=$snipped]$end", length $data);
+}
+    
+###########################################
+sub lenformat {
+###########################################
+    my($data, $orglen) = @_;
+
+    return "(" . ($orglen || length($data)) . ")[" .
+        printable($data) . "]";
+}
+
+###########################################
+sub printable {
+###########################################
+    my($data) = @_;
+
+    $data =~ s/[^ \w.;!?@#$%^&*()+\\|~`'-,><[\]{}="]/./g;
+    return $data;
+}
+
+=pod
+
 =back
 
 =head1 AUTHOR
